//
// Jupiter
// (C) 2007-2011 Andrew Wyatt
// Distributed under GNU GPL v2

using Gtk;
using Gdk;
using System;
using Mono.Unix;
using System.IO;
using System.Timers;
using System.Text.RegularExpressions; 
using System.Reflection;

[assembly:AssemblyTitleAttribute ("Jupiter")]
[assembly:AssemblyDescriptionAttribute ("Generic interface for display, power, and device control.")]
[assembly:AssemblyCopyrightAttribute("Copyright 2009-2011 - Andrew Wyatt (Fewt)")]


class jupiter {
	private static StatusIcon trayIcon;
	private static Gtk.Window window;
	private static System.Timers.Timer Clock;
	private static int ClkRate = 2000;
        private static string IconPath = "/usr/share/pixmaps/";

	private static string MyName = "Jupiter";
	private static string MyVersion = "0.0.52";

	private static string Path = "/usr/lib/jupiter/scripts";
	private static string VendorPath = "/usr/lib/jupiter/vendors";
	private static string Var = "/var/jupiter";

	private static bool SWDebug = false;

	private static string Temp = "/sys/devices/virtual/hwmon/hwmon0/temp1_input";
	private static string CPUFreq = "/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor";

	private static string CurrentCPUMode = Var + "/cpu_mode";

        private static string BT_Available = CheckState( Path + "/bluetooth" );
        private static string WIFI_Available = CheckState( Path + "/wifi" );
	private static string TP_Available = CheckState( Path + "/touchpad" );

	private static string VendorCPUFreq = GetVendor();

	private static string MyIconName = null;

	static string GetVideoModes () {
                try {
			if ( SWDebug == true ) {
				Console.WriteLine(Catalog.GetString ("Collecting available video modes."));

			}
                        System.Diagnostics.ProcessStartInfo State = new System.Diagnostics.ProcessStartInfo();
			State.FileName = "sudo";
			State.Arguments = Path + "/resolutions modes";
                        State.UseShellExecute = false;
                        State.ErrorDialog = false;
                        State.CreateNoWindow = true;
                        State.RedirectStandardOutput = true;
                        System.Diagnostics.Process  GetState = System.Diagnostics.Process.Start(State);
                        GetState.WaitForExit();
                        State = null;
                        return "";
                }
                catch (Exception) {
                        Console.WriteLine(Catalog.GetString ("Unable to capture available video modes."));
                        return "";
                }

	}

	static void JupiterAbout() {
                if ( SWDebug == true ) {
			Console.WriteLine(Catalog.GetString ("Building About dialog"));
		}
		AboutDialog dialog = new AboutDialog ();

		Assembly asm = Assembly.GetExecutingAssembly ();
		dialog.ProgramName = (asm.GetCustomAttributes (
			typeof (AssemblyTitleAttribute), false) [0]
			as AssemblyTitleAttribute).Title;

		dialog.Logo = (new Gdk.Pixbuf(IconPath + "jupiter.png"));
		dialog.Version = MyVersion;
		dialog.Comments = (asm.GetCustomAttributes (
			typeof (AssemblyDescriptionAttribute), false) [0]
			as AssemblyDescriptionAttribute).Description;

		dialog.Copyright = (asm.GetCustomAttributes (
			typeof (AssemblyCopyrightAttribute), false) [0]
			as AssemblyCopyrightAttribute).Copyright;

		dialog.Website = Catalog.GetString("http://www.jupiterapplet.org");
		dialog.WebsiteLabel = Catalog.GetString("Homepage");

		dialog.Run();
		dialog.Destroy();

	}

        static string GetVendor () {
                if ( SWDebug == true ) {
		  Console.WriteLine(Catalog.GetString ("Getting Vendor information."));
		}
                try {
                        System.Diagnostics.ProcessStartInfo State = new System.Diagnostics.ProcessStartInfo();
			State.FileName = "sudo";
			State.Arguments = Path + "/cpu-control vendor";
                        State.UseShellExecute = false;
                        State.ErrorDialog = false;
                        State.CreateNoWindow = true;
                        State.RedirectStandardOutput = true;
                        System.Diagnostics.Process GetState = System.Diagnostics.Process.Start(State);
                        System.IO.StreamReader Results = GetState.StandardOutput;
                        string ToSend = Results.ReadToEnd();
                        GetState.WaitForExit();
                        Results = null;
                        GetState = null;
                        State = null;
                        ToSend = CleanData(ToSend);
                        return ToSend;
                }
                catch (Exception) {
                        Console.WriteLine(Catalog.GetString ("Unable to get vendor information for this system."));
                        return "";
                }

        }


	static string CheckState (string cli) {
                if ( SWDebug == true ) {
		  Console.WriteLine(Catalog.GetString ("Checking the state of [" + cli + "]"));
		}
		try {
			System.Diagnostics.ProcessStartInfo State = new System.Diagnostics.ProcessStartInfo(cli,"status");
			State.UseShellExecute = false; 
			State.ErrorDialog = false; 
			State.CreateNoWindow = true; 
			State.RedirectStandardOutput = true; 
			System.Diagnostics.Process GetState = System.Diagnostics.Process.Start(State);
			System.IO.StreamReader Results = GetState.StandardOutput;
			string ToSend = Results.ReadToEnd();
            GetState.WaitForExit();
			Results = null;
			GetState = null;
			State = null;
			ToSend = CleanData(ToSend);
			return ToSend;
		}
		catch (Exception) {
			Console.WriteLine(Catalog.GetString ("Unable to check state of [" + cli + "]"));
			return "";
		}

	}

	static void Exec (string command, string args) {
               if ( SWDebug == true ) {
                  Console.WriteLine(Catalog.GetString ("Setting [" + command + "] to [" + args + "]"));
                }
                try {
                     System.Diagnostics.ProcessStartInfo State = new System.Diagnostics.ProcessStartInfo();
                     State.FileName = "sudo";
                     State.Arguments = Path + "/" + command + " " + args;
                     State.UseShellExecute = false;
                     State.ErrorDialog = false;
                     State.CreateNoWindow = true;
                     State.RedirectStandardOutput = true;
                     System.Diagnostics.Process GetState = System.Diagnostics.Process.Start(State);
                     GetState.WaitForExit();
                     State = null;
		}
                catch (Exception) {
                        Console.WriteLine(Catalog.GetString ("Unable to set [" + command + "] to [" + args + "]"));
                }

        }

	static string SwitchState (string args) {
                if ( SWDebug == true ) {
		  Console.WriteLine(Catalog.GetString ("Switching mode [" + args + "]"));
		}
		if ( args == "OFF" ) {
			args = "Enable";
		} else if ( args == "ON" ) {
			args = "Disable";
		}
                if ( args == "0" ) {
			args = "Enable";
		} else if ( args == "1" ) {
			args = "Disable";
		}
		return args;
	}

	static string CleanData (string args) {
		Regex CleanUp = new Regex("\n$");
		args = CleanUp.Replace(args,"");
		CleanUp=null;
		return args;
	}

	static string CollectData (string args) {
                if ( SWDebug == true ) {
		  Console.WriteLine(Catalog.GetString ("Collecting information about [" + args + "]"));
		}
		try {
			using (StreamReader MyInput = File.OpenText(args)) {
				string input = null;
				input = MyInput.ReadToEnd();
				input = CleanData(input);
				return input;
			}
		}
		catch (Exception) {
			return "UNKNOWN";
		}
	}

	static string GetCPUMode (string args) {
		string MyMode = CollectData(CurrentCPUMode);
		Regex ModeChk = new Regex(args);

		if ( ModeChk.IsMatch(MyMode) ) {
		  ModeChk=null;
                  return " ***";
                } else {
		  ModeChk=null;
                  return "";
		}
	}

        static string Get_Current_Rotation (string args) {
                if ( SWDebug == true ) {
		  Console.WriteLine(Catalog.GetString ("Getting current rotation."));
		}
                string MyMode = CollectData(Var + "/rotation_saved");
                Regex ModeChk = new Regex(args);

                if ( ModeChk.IsMatch(MyMode) ) {
                  ModeChk=null;
                  return " ***";
                } else {
                  ModeChk=null;
                  return "";
                }
        }

       static string Get_Current_Display (string args) {
                if ( SWDebug == true ) {
		  Console.WriteLine(Catalog.GetString ("Getting display information."));
		}
                string MyMode = CollectData(Var + "/vga_saved");
                Regex ModeChk = new Regex(args);

                if ( ModeChk.IsMatch(MyMode) ) {
                  ModeChk=null;
                  return " ***";
                } else {
                  ModeChk=null;
                  return "";
                } 
	}

        static string ReportFullCPUMode () {
                string MyMode = CollectData(CurrentCPUMode);
                if ( MyMode == "super" ) {
                  return Catalog.GetString ("Maximum Performance");
                }
                if ( MyMode == "high" ) {
		  return Catalog.GetString ("Power On Demand");
		}
		if ( MyMode == "powersave" ) {
		  return Catalog.GetString ("Power Savings");
		} else {
		  return "Unknown Mode";
		}
        }

	static string GetTemp (string args) {
		string MyTemp = CollectData(args);
		Regex ParseTemp = new Regex("000");
		MyTemp = ParseTemp.Replace(MyTemp,"");
		ParseTemp=null;
		return MyTemp;
	}

	static void SetTrayIcon(string args) {
		Gtk.Application.Invoke(delegate{
			string Icon = args;
			Regex NoExt = new Regex("\\.[A-z]*");
			Icon = NoExt.Replace(Icon,"");
			trayIcon.IconName = (Icon);
		});
	}

	static void Main() {
		Application.Init ();
	        Catalog.Init("i18n","./locale");
		GetVideoModes();
		window = new Gtk.Window (MyName);
		window.DeleteEvent += delegate { Application.Quit(); };
		trayIcon = new StatusIcon();
		// trayIcon.IconName = ("bolt2.png");
		SetTrayIcon("bolt2.png");
                trayIcon.Activate += OnTrayRtClk;
                trayIcon.PopupMenu += OnTrayRtClk;

                Clock = new System.Timers.Timer();
                Clock.Elapsed += new ElapsedEventHandler(OnTimerElapsed);
                Clock.Interval = ClkRate;
                Clock.Enabled = true;

		trayIcon.Tooltip = MyName + Catalog.GetString (" - Generic interface for display,\npower, and device control.");

		string StartCPUMode = CollectData(CurrentCPUMode);
		if ( StartCPUMode == "UNKNOWN" ) {
		  Gtk.Application.Invoke( delegate { Exec("cpu-control","initial"); });
		}

		string StartVGAMode = CollectData( Var + "/vga_saved");
		if ( StartVGAMode == "UNKNOWN" ) {
		  Gtk.Application.Invoke( delegate { Exec("vga-out","clone"); });
		}

		string StartResolutions = CollectData( Var + "/rotation_saved");
		if ( StartResolutions == "UNKNOWN" ) {
		  Gtk.Application.Invoke( delegate { Exec("rotate","normal"); });
		}

		Application.Run ();
	}

	static void OnTimerElapsed(object sender, ElapsedEventArgs e) {
		string TrayTemperature = GetTemp(Temp);
		string TrayCPUMode = ReportFullCPUMode();
		TrayCPUMode = Catalog.GetString ("CPU Mode: ") + TrayCPUMode;

	        if (File.Exists(CPUFreq) == true || File.Exists(VendorPath + "/" + VendorCPUFreq + "/cpu-control") == true) {
	                Regex ModeChk = new Regex(CollectData(CurrentCPUMode));
	               	if ( ModeChk.IsMatch("super") ) {
       	                 	if ( MyIconName != "super") {
					SetTrayIcon("bolt2.png");
					MyIconName = "super";
				}
			} else if ( ModeChk.IsMatch("high") ) {
                        	if ( MyIconName != "high") {
					SetTrayIcon("bolt1.png");
					MyIconName = "high";
				}
			} else if ( ModeChk.IsMatch("powersave") ) {
                	        if ( MyIconName != "powersave") {
					SetTrayIcon("bolt4.png");
					MyIconName = "powersave";
				}
			} else {
                	        if ( MyIconName != "default") {
					SetTrayIcon("jupiter.png");
					MyIconName = "default";
				}
			}
		} else {
			TrayCPUMode = "";
		}
               	Gtk.Application.Invoke(delegate{
			if ( TrayTemperature != "UNKNOWN" ) {
                     		TrayTemperature=Catalog.GetString ("\nTemperature: ") + TrayTemperature + " C";
			} else {
			 	TrayTemperature="";
			}
			trayIcon.Tooltip = TrayCPUMode + TrayTemperature;
               	});
	}

	static void OnTrayRtClk (object o, EventArgs args) {
                Menu RpopupMenu = new Menu();

                if (File.Exists(CPUFreq) == true  || File.Exists(VendorPath + "/" + VendorCPUFreq + "/cpu-control") == true) {
	      		Menu PerfMenu = new Menu();
	      		ImageMenuItem PerfMenuItem = new ImageMenuItem(Catalog.GetString ("Performance _Modes"));

	      		Gdk.Pixbuf pfImg = new Gdk.Pixbuf(IconPath + "jupiter.png");
	      		PerfMenuItem.Image = new Gtk.Image (pfImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));

	      		PerfMenuItem.Submenu = PerfMenu;
	      		RpopupMenu.Add(PerfMenuItem);

			// Super High
			ImageMenuItem performance = new ImageMenuItem (Catalog.GetString ("_Maximum Performance") + GetCPUMode("super"));
	                performance.Image = new Gtk.Image(new Gdk.Pixbuf(IconPath + "performance.png"));
			PerfMenu.Add(performance);
	                performance.Activated += delegate { Exec("cpu-control","super"); };

	                // High
	                ImageMenuItem normal = new ImageMenuItem (Catalog.GetString ("_High Performance") + GetCPUMode("high"));
	                normal.Image = new Gtk.Image(new Gdk.Pixbuf(IconPath + "ondemand.png"));
	                PerfMenu.Add(normal);
	                normal.Activated += delegate { Exec("cpu-control","high"); };

			// Powersave
			ImageMenuItem powersave = new ImageMenuItem (Catalog.GetString ("_Power Saver") + GetCPUMode("powersave"));
			powersave.Image = new Gtk.Image(new Gdk.Pixbuf(IconPath + "powersave.png"));
			PerfMenu.Add(powersave);
	                powersave.Activated += delegate { Exec("cpu-control","powersave"); };

			// Seperator
	                ImageMenuItem menuItemDash1 = new ImageMenuItem ();
	                RpopupMenu.Add(menuItemDash1);
		}

      		Menu DevMenu = new Menu();
      		ImageMenuItem DevMenuItem = new ImageMenuItem(Catalog.GetString ("_Device Control"));

      		Gdk.Pixbuf dvImg = new Gdk.Pixbuf(IconPath + "devices.png");
      		DevMenuItem.Image = new Gtk.Image (dvImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));
 
      		DevMenuItem.Submenu = DevMenu;

		int DevMenuDevices = 0;

		// Wifi
                if ((WIFI_Available != "UNKNOWN" ) && (WIFI_Available != "-1")) {
			string WifiStatus = CollectData ( Var + "wifi-saved" );
                	if ( WifiStatus == "UNKNOWN" ) {
				WifiStatus = CheckState( Path + "/wifi");
			}
			if ( WifiStatus != "UNKNOWN" ) {
				WifiStatus = SwitchState(WifiStatus);
                		ImageMenuItem menuItemWIFI = new ImageMenuItem (Catalog.GetString (WifiStatus + " WI_FI "));
                		Gdk.Pixbuf wifiImg = new Gdk.Pixbuf(IconPath + "gnome-dev-wavelan.png");
                		menuItemWIFI.Image= new Gtk.Image (wifiImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));
                		DevMenu.Add(menuItemWIFI);
                		menuItemWIFI.Activated += delegate { Exec("wifi","toggle"); };
				DevMenuDevices++;
			}
		}

		// Bluetooth
		if ((BT_Available != "UNKNOWN" ) && (BT_Available != "-1")) {
			string BluetoothStatus = CollectData ( Var + "bt_saved" );
			if ( BluetoothStatus == "UNKNOWN" ) {
				BluetoothStatus = CheckState( Path + "/bluetooth");
			}
			BluetoothStatus = SwitchState(BluetoothStatus);
			if ( BluetoothStatus != "UNKNOWN" ) {
                		ImageMenuItem menuItemBT = new ImageMenuItem (Catalog.GetString (BluetoothStatus + " _Bluetooth "));
                		Gdk.Pixbuf btImg = new Gdk.Pixbuf(IconPath + "bluetooth.png");
                		menuItemBT.Image = new Gtk.Image (btImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));
                		DevMenu.Add(menuItemBT);
                		menuItemBT.Activated += delegate { Exec("bluetooth","toggle"); };
				DevMenuDevices++;
			}
		}

		// Touchpad
		if ( TP_Available != "UNAVAILABLE" ) {
		  string TouchpadStatus = CollectData ( Var + "touchpad_saved" );
		  if ( TouchpadStatus == "UNKNOWN" ) {
		  	TouchpadStatus = CheckState( Path + "/touchpad");
	  	  }
		  TouchpadStatus = SwitchState(TouchpadStatus);
                  ImageMenuItem menuItemTP = new ImageMenuItem (Catalog.GetString (TouchpadStatus + " _Touchpad "));
                  Gdk.Pixbuf tpImg = new Gdk.Pixbuf(IconPath + "input-tablet.png");
                  menuItemTP.Image = new Gtk.Image (tpImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));
                  DevMenu.Add(menuItemTP);
                  menuItemTP.Activated += delegate { Exec("touchpad","toggle"); };
		  DevMenuDevices++;
                }
                if ( SWDebug == true ) {
		  Console.WriteLine (Catalog.GetString ("Devices Detected: [" + DevMenuDevices + "]"));
		}
		if ( DevMenuDevices >= 1 ) {
      		  RpopupMenu.Add(DevMenuItem);
		  // Seperator
                  ImageMenuItem menuItemDash5 = new ImageMenuItem ();
                  RpopupMenu.Add(menuItemDash5);
		}

		// Cycle Resolutions
		if(File.Exists(Var + "/available_resolutions") == false) {
			try { 
				Exec("resolution","modes");
			}
			catch (Exception) {
				Console.WriteLine (Catalog.GetString ("Unable to collect available resolutions"));
			}
		}
		try {
			string[] Resolutions = CollectData(Var + "/available_resolutions").Split(' ');
                	Menu ResolutionMenu = new Menu();
                	ImageMenuItem ResMenuItem = new ImageMenuItem(Catalog.GetString ("_Screen Resolutions"));

                	Gdk.Pixbuf rsImg = new Gdk.Pixbuf(IconPath + "resolution.png");
                	ResMenuItem.Image = new Gtk.Image (rsImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));

                	ResMenuItem.Submenu = ResolutionMenu;
                	RpopupMenu.Add(ResMenuItem);

			if ( Resolutions[1] != "" ) {
                	  string ResObj0Name = Resolutions[1];
                	  if ( Resolutions[1] == Resolutions[0] ) {
                	    ResObj0Name = ResObj0Name + " ***";
			  }
                	  ImageMenuItem ResObj0 = new ImageMenuItem (ResObj0Name);
                	  ResObj0.Activated += delegate { Exec("resolutions",Resolutions[1]); };
       		           ResolutionMenu.Add(ResObj0);
			}

        	        if ( Resolutions[2] != "" ) {
	                 string ResObj1Name = Resolutions[2];
                	  if ( Resolutions[2] == Resolutions[0] ) {
        	            ResObj1Name = ResObj1Name + " ***";
	                  }
                	  ImageMenuItem ResObj1 = new ImageMenuItem (ResObj1Name);
        	          ResObj1.Activated += delegate { Exec("resolutions",Resolutions[2]); };
	                  ResolutionMenu.Add(ResObj1);
        	        }

	                if ( Resolutions[3] != "" ) {
	                 string ResObj2Name = Resolutions[3];
	                  if ( Resolutions[3] == Resolutions[0] ) {
	                    ResObj2Name = ResObj2Name + " ***";
	                  }
	                  ImageMenuItem ResObj2 = new ImageMenuItem (ResObj2Name);
	                  ResObj2.Activated += delegate { Exec("resolutions",Resolutions[3]); };
	                  ResolutionMenu.Add(ResObj2);
	                }

                        if ( Resolutions[4] != "" ) {
                         string ResObj3Name = Resolutions[4];
                          if ( Resolutions[4] == Resolutions[0] ) {
                            ResObj3Name = ResObj3Name + " ***";
                          }
                          ImageMenuItem ResObj3 = new ImageMenuItem (ResObj3Name);
                          ResObj3.Activated += delegate { Exec("resolutions", Resolutions[4]); };
                          ResolutionMenu.Add(ResObj3);
                        }
 
                }
                catch (Exception) {
                  Console.WriteLine(Catalog.GetString ("Can not retreive resolutions"));
                }

		// Rotate
      		Menu RotateMenu = new Menu();
      		ImageMenuItem RotateMenuItem = new ImageMenuItem(Catalog.GetString ("Screen _Orientation"));

      		Gdk.Pixbuf rtImg = new Gdk.Pixbuf(IconPath + "rotate.png");
      		RotateMenuItem.Image = new Gtk.Image (rtImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));
 
      		RotateMenuItem.Submenu = RotateMenu;
      		RpopupMenu.Add(RotateMenuItem);

                ImageMenuItem menuItemRTN = new ImageMenuItem (Catalog.GetString ("_Normal") + Get_Current_Rotation("normal"));
                RotateMenu.Add(menuItemRTN);
                menuItemRTN.Activated += delegate { Exec("rotate","normal"); };

		ImageMenuItem menuItemRTL = new ImageMenuItem (Catalog.GetString ("_Left") + Get_Current_Rotation("left"));
                RotateMenu.Add(menuItemRTL);
                menuItemRTL.Activated += delegate { Exec("rotate","left"); };

                ImageMenuItem menuItemRTR = new ImageMenuItem (Catalog.GetString ("_Right") + Get_Current_Rotation("right"));
                RotateMenu.Add(menuItemRTR);
                menuItemRTR.Activated += delegate { Exec("rotate","right"); };

                ImageMenuItem menuItemRTI = new ImageMenuItem (Catalog.GetString ("_Upside Down") + Get_Current_Rotation("inverted"));
                RotateMenu.Add(menuItemRTI);
                menuItemRTI.Activated += delegate { Exec("rotate","inverted"); };

		// VGA on/off
         
                Menu VGAMenu = new Menu();
                ImageMenuItem VGAMenuItem = new ImageMenuItem(Catalog.GetString ("_Video Displays"));

                Gdk.Pixbuf vgaImg = new Gdk.Pixbuf(IconPath + "display.png");
                VGAMenuItem.Image = new Gtk.Image (vgaImg.ScaleSimple (16, 16, Gdk.InterpType.Bilinear));
 
                VGAMenuItem.Submenu = VGAMenu;
                RpopupMenu.Add(VGAMenuItem);

       		ImageMenuItem menuItemVGAE = new ImageMenuItem (Catalog.GetString ("_External Display Only") + Get_Current_Display("vga"));
                VGAMenu.Add(menuItemVGAE);
                menuItemVGAE.Activated += delegate {
Exec("vga-out","vga");
};

       		ImageMenuItem menuItemVGAI = new ImageMenuItem (Catalog.GetString ("_Internal Display Only") + Get_Current_Display("lvds"));
                VGAMenu.Add(menuItemVGAI);
                menuItemVGAI.Activated += delegate { Exec("vga-out","lvds"); };

       		ImageMenuItem menuItemVGAC = new ImageMenuItem (Catalog.GetString ("Enable _Both Displays") + Get_Current_Display("clone"));
                VGAMenu.Add(menuItemVGAC);
                menuItemVGAC.Activated += delegate {
Exec("vga-out","clone");
};

		// Seperator
                ImageMenuItem menuItemDash2 = new ImageMenuItem ();
                RpopupMenu.Add(menuItemDash2);

		// About
		ImageMenuItem menuItemAbout = new ImageMenuItem (Catalog.GetString ("_About"));
		Gtk.Image aboimg = new Gtk.Image (Stock.About, IconSize.Menu);
		menuItemAbout.Image = aboimg;
		RpopupMenu.Add(menuItemAbout);
		menuItemAbout.Activated += delegate { JupiterAbout(); };

		// Exit
                ImageMenuItem menuItemQuit = new ImageMenuItem (Catalog.GetString ("E_xit"));
                Gtk.Image appimg = new Gtk.Image (Stock.Quit, IconSize.Menu);
                menuItemQuit.Image = appimg;
                RpopupMenu.Add(menuItemQuit);
                menuItemQuit.Activated += delegate { Application.Quit(); };
                RpopupMenu.ShowAll();
                RpopupMenu.Popup();
		RpopupMenu=null;
        }

}
